@@ -1,46 +1,61 @@
 import datetime
 import os
 from enum import Enum
-from io import BytesIO, RawIOBase
-from typing import IO, Any, Dict, List, Optional, Union, cast
+from io import BytesIO
+import json
+from typing import IO, Any, Dict, List, Optional, Union
 
 import arrow
 import requests
 from typing_extensions import Protocol
 
+
 try:
-    from mcap.mcap0.records import Schema as McapSchema
-    from mcap.mcap0.stream_reader import StreamReader as McapStreamReader
+    from mcap.records import Schema as McapSchema
+    from mcap.reader import make_reader
 except ModuleNotFoundError:
     McapSchema = None
-    McapStreamReader = None
+    make_reader = None
+
+
+def _err_on_construction(err):
+    def construct():
+        raise RuntimeError(f"Error importing decoder implementation: {err}")
+
+    return construct
+
 
 try:
     from mcap_ros1.decoder import Decoder as Ros1Decoder
-except ModuleNotFoundError:
-    Ros1Decoder = None
+except ModuleNotFoundError as err:
+    Ros1Decoder = _err_on_construction(err)
 
 try:
     from mcap_protobuf.decoder import Decoder as ProtobufDecoder
-except ModuleNotFoundError:
-    ProtobufDecoder = None
+except ModuleNotFoundError as err:
+    ProtobufDecoder = _err_on_construction(err)
+
+try:
+    from mcap_ros2.decoder import Decoder as Ros2Decoder
+except ModuleNotFoundError as err:
+    Ros2Decoder = _err_on_construction(err)
 
 
-def decoder_for_schema(schema: Any):
-    if not McapSchema:
-        return None
-    if isinstance(schema, McapSchema) and schema.encoding == "ros1msg":
-        if not Ros1Decoder:
-            raise Exception(
-                "Mcap ROS1 library not found. Please install the mcap-ros1-support library."
-            )
-        return Ros1Decoder
-    if isinstance(schema, McapSchema) and schema.encoding == "protobuf":
-        if not ProtobufDecoder:
-            raise Exception(
-                "Mcap protobuf library not found. Please install the mcap-protobuf-support library."
-            )
-        return ProtobufDecoder
+class JsonDecoder:
+    def decode(self, schema_, message):
+        return json.loads(message.data.decode("utf-8"))
+
+
+def decoder_for_schema_encoding(encoding_string):
+    if encoding_string == "ros1msg":
+        return Ros1Decoder()
+    if encoding_string == "ros2msg":
+        return Ros2Decoder()
+    if encoding_string == "protobuf":
+        return ProtobufDecoder()
+    if encoding_string == "jsonschema":
+        return JsonDecoder()
+    raise RuntimeError(f"No known decoder class for encoding {encoding_string}")
 
 
 def camelize(snake_name: Optional[str]) -> Optional[str]:
@@ -122,15 +137,16 @@ def json_or_raise(response: requests.Response):
 
 
 class Client:
-    def __init__(self, token: str):
+    def __init__(self, token: str, host: str = "api.foxglove.dev"):
         self.__token = token
         self.__headers = {
             "Content-type": "application/json",
             "Authorization": "Bearer " + self.__token,
         }
+        self.__host = host
 
     def __url__(self, path: str):
-        return f"https://api.foxglove.dev{path}"
+        return f"https://{self.__host}{path}"
 
     def create_event(
         self,
@@ -260,31 +276,32 @@ class Client:
         """
         Returns a list of tuples of (topic, raw mcap record, decoded message).
 
-        This will throw an exception if an appropriate message decoder can't be found.
-
         device_id: The id of the device that originated the desired data.
         start: The earliest time from which to retrieve data.
         end: The latest time from which to retrieve data.
         topics: An optional list of topics to retrieve.
             All topics will be retrieved if this is omitted.
         """
-        if not McapSchema or not McapStreamReader:
-            raise Exception("Mcap library not found. Please install the mcap library.")
+        if not McapSchema or not make_reader:
+            raise RuntimeError(
+                "Mcap library not found. Please install the mcap library."
+            )
         data = self.download_data(
             device_id=device_id, start=start, end=end, topics=topics
         )
-        reader = McapStreamReader(cast(RawIOBase, BytesIO(data)))
-        decoder = None
-        for r in reader.records:
-            decoder = decoder_for_schema(r)
-            if decoder:
-                break
-        if not decoder:
-            raise Exception("Unknown mcap file encoding encountered.")
-        return [
-            m
-            for m in decoder(McapStreamReader(cast(RawIOBase, BytesIO(data)))).messages
-        ]
+        reader = make_reader(BytesIO(data))
+        decoders = {}
+        output_messages = []
+        for schema, channel, message in reader.iter_messages():
+            if schema.encoding not in decoders:
+                decoder = decoder_for_schema_encoding(schema.encoding)
+                decoders[schema.encoding] = decoder
+            else:
+                decoder = decoders[schema.encoding]
+            output_messages.append(
+                (channel.topic, message, decoder.decode(schema, message))
+            )
+        return output_messages
 
     def download_data(
         self,
